import logging
import pathlib

import dict2xml
import xmltodict


class XmlHandler:
    """
    XML files handling class
    """

    def __init__(self, xml_file_path: pathlib.Path):
        """
        Constructor

        Parameters:
            xml_file_path (pathlib.Path) : Path to Pascal VOC annotation XML file

        Throws:
            Exception if file does not exist
        """
        logging.debug("XmlHandler: XmlHandler initiated")
        if not xml_file_path.exists():
            raise Exception(f"The file {xml_file_path} does not exist")

        self.xml_file_path = xml_file_path
        self.xml_data_dict = self._xml_to_dict(xml_file_path)
        self.bounding_boxes_count = self._get_bounding_boxes_count()

    def get_xml_file_name(self, include_extension: bool = True) -> str:
        """
        Get the xml file name

        Parameters:
            include_extension: Whether to include image extension in file name
                Default=True
                Eg. Image name = image1.xml
                    Include extension = True, returns image1.xml
                    Include extension = False, returns image1
        """
        if include_extension:
            return self.xml_file_path.name
        return self.xml_file_path.stem

    def get_xml_file_extension(self, include_dot: bool = False) -> str:
        """
        Get the XML file extension

        Parameters:
            include_dot: Whether to return "xml" or ".xml"
                Default=False

        Eg. "image1.xml" returns "xml", if include dot is True returns ".xml"
            "image1.new.xml" returns "xml", if include dot is True returns ".xml"
        """
        ext = ""
        if include_dot:
            ext = "."
        return ext + self.xml_file_path.name.rsplit(".", 1)[-1]

    def _get_bounding_boxes_count(self) -> int:
        """
        Returns the count of objects/labels present in XML file
        """
        if "object" in self.xml_data_dict["annotation"]:
            annotated_objects = self.xml_data_dict["annotation"]["object"]
            if isinstance(annotated_objects, dict):
                # Single object
                return 1
            # Multiple objects
            return len(annotated_objects)
        return 0

    def get_bounding_boxes_count(self) -> int:
        """
        Get the count of objects/labels present in XML file
        """
        return self.bounding_boxes_count

    def _xml_to_dict(self, xml_file_path: str) -> dict:
        """
        Private method to convert XML file into a python dictionary

        Parameters:
            xml_file_path (str): Complete path to XML file to be loaded
        """
        with open(xml_file_path, 'r', encoding="utf-8") as xml_file:
            xml_file_data = xml_file.read()

        return xmltodict.parse(xml_file_data)

    def get_all_bbox_coordinates_list(self) -> list[list]:
        """
        Get list of objects with their coordinates in form of a 2D list
        Coordinates are: [xmin, ymin, xmax, ymax, label]

        Eg. [
            [1650, 418, 1668, 455, "person"],
            ...
        ]

        Return 2D list of all bounding box coordinates,
        if no bounding box found, returns empty list []
        """
        logging.debug("XmlHandler: Creating bounding boxes coordinates list")
        bounding_box_coordinates_list = []

        if self.bounding_boxes_count > 0:
            bounding_boxes = self.xml_data_dict["annotation"]["object"]

        if self.bounding_boxes_count == 1:
            # Single object/labels/annotation bounding box found
            # Object is of type dictionary, because only single object is present
            bounding_box_coordinates_list.append(self._get_bbox_coordinate(bounding_boxes))
        elif self.bounding_boxes_count > 1:
            # Multiple objects/labels/annotation boudning boxes found
            # Object is of type list, because multiple objects are present
            for bounding_box in bounding_boxes:
                bounding_box_coordinates_list.append(self._get_bbox_coordinate(bounding_box))

        return bounding_box_coordinates_list

    def _get_bbox_coordinate(self, bbox_dict: dict) -> list:
        """
        Get list of label/object/boudning box coordinates
        Coordinates are: [xmin, ymin, xmax, ymax, label]

        Parameters:
            bbox_dict(dict): Boudning box data as dict type

        Return a list of bounding box coordinate
        """
        xmin = int(float(bbox_dict["bndbox"]["xmin"]))
        ymin = int(float(bbox_dict["bndbox"]["ymin"]))
        xmax = int(float(bbox_dict["bndbox"]["xmax"]))
        ymax = int(float(bbox_dict["bndbox"]["ymax"]))
        label = bbox_dict["name"]
        return [xmin, ymin, xmax, ymax, label]

    def update_all_bounding_box_coordinates(self, bounding_boxes_coordinates_list: list[list]) -> None:
        """
        Update the bounding box coordinates in the XML file

        Parameters:
            bounding_boxes_coordinates_list: A 2D list containing all bouding box coordinates
                Format- [xmin, ymin, xmax, ymax, label]
                Eg.- [
                    [1650, 418, 1668, 455, "person"],
                    ...
                ]
        """
        if self.bounding_boxes_count > 0:
            annotated_object = self.xml_data_dict["annotation"]["object"]

            if self.bounding_boxes_count == 1:
                self._update_bbox_coordinate(annotated_object, bounding_boxes_coordinates_list[0])
            elif self.bounding_boxes_count > 1:
                i = 0
                for bounding_box in annotated_object:
                    self._update_bbox_coordinate(bounding_box, bounding_boxes_coordinates_list[i])
                    i += 1

    def _update_bbox_coordinate(self, bbox_dict: dict, bbox_coordinate: list) -> None:
        """
        Update bounding box coordinates and name in object/label

        Parameters:
            bbox_dict: A dict containing boudning box details, this dict will be updated
                with new bounding box coordinates
            bbox_coordinates: New coordinates for the bounding box
                Coordinate list format: [xmin, ymin, xmax, ymax, name]
        """
        bbox_dict["bndbox"]["xmin"] = bbox_coordinate[0]
        bbox_dict["bndbox"]["ymin"] = bbox_coordinate[1]
        bbox_dict["bndbox"]["xmax"] = bbox_coordinate[2]
        bbox_dict["bndbox"]["ymax"] = bbox_coordinate[3]
        bbox_dict["name"] = bbox_coordinate[4]

    def get_annotation_classes_details(self) -> dict:
        """
        Returns a dict of all classes present in XML file with their details
        Eg: {
            "<class_name>": <number_of_occurences>
        }
        """
        objects_dict = {}

        if self.bounding_boxes_count > 0:
            annotated_objects = self.xml_data_dict["annotation"]["object"]

        if self.bounding_boxes_count == 1:
            # Single object present
            obj_name = annotated_objects["name"]
            if obj_name in objects_dict:
                objects_dict[obj_name] += 1
            else:
                objects_dict[obj_name] = 1
        elif self.bounding_boxes_count > 1:
            # Multiple objects present
            for obj in annotated_objects:
                obj_name = obj["name"]
                if obj_name in objects_dict:
                    objects_dict[obj_name] += 1
                else:
                    objects_dict[obj_name] = 1

        return objects_dict

    def remove_class(self, remove_class_list: list) -> None:
        """
        To remove object of specific classes from the annotation file

        Parameters:
            class_list (list): List of class labels to be removed from the annotation(.xml) file
        """
        if self.bounding_boxes_count > 0:
            # List of classes available in current annotation file
            available_class_list = list(self.get_annotation_classes_details())

            # Check for any removable class
            is_removable_class_present = False
            for class_name in available_class_list:
                if class_name in remove_class_list:
                    is_removable_class_present = True

            if is_removable_class_present:
                # Removing the required classes
                annotated_objects = self.xml_data_dict["annotation"]["object"]
                filtered_objects = []

                if self.bounding_boxes_count == 1 and (annotated_objects["name"] not in remove_class_list):
                    # There is only 1 annotation in the current file and need to be removed
                    filtered_objects.append(annotated_objects)
                elif self.bounding_boxes_count > 1:
                    # Multiple objects in file
                    for obj in annotated_objects:
                        obj_name = obj["name"]
                        if obj_name not in remove_class_list:
                            filtered_objects.append(obj)

                # Updating annotation file
                if len(filtered_objects) == 0:
                    # No object of interest present, remove object tag
                    self.xml_data_dict["annotation"].pop("object")
                if len(filtered_objects) == 1:
                    # Only single object of ineterest present, adding object as dict
                    self.xml_data_dict["annotation"]["object"] = filtered_objects[0]
                elif len(filtered_objects) > 1:
                    # Multiple objects of interest present, adding objects as a list
                    self.xml_data_dict["annotation"]["object"] = filtered_objects

        # Updating objects count after removing objects/annotations
        self.bounding_boxes_count = self._get_bounding_boxes_count()

    def save_xml_file(self, save_path: str) -> None:
        """
        Save the XML file

        Limitation:
            Not able to add attribubtes, so this will remove verified attribute

        Parameters:
            save_path (str): Complete path with extension of XML file
        """
        logging.debug("XmlHandler: Save annotation file %s", save_path)

        # Due to library limitation removing verified attribute from annotation tag
        # https://github.com/delfick/python-dict2xml#limitations
        if "@verified" in self.xml_data_dict["annotation"]:
            self.xml_data_dict["annotation"].pop("@verified")

        new_xml_data = dict2xml.dict2xml(self.xml_data_dict)
        xml_file = open(save_path, "w", encoding="utf-8")
        xml_file.write(new_xml_data)
        xml_file.close()



import pathlib

from .image_handler import ImageHandler
from .xml_handler import XmlHandler


class PascalVocDatasetInstance:
    def __init__(self, image_file_path: pathlib.Path, annotation_file_path: pathlib.Path) -> None:
        self.image_file_path = image_file_path
        self.annotation_file_path = annotation_file_path

        if not self.image_file_path.exists():
            raise Exception(f"Specified image file does not exist: {self.image_file_path}")

        if not self.annotation_file_path.exists():
            raise Exception(f"Specified annotation file does not exist: {self.annotation_file_path}")

        self.xml_handler = XmlHandler(self.annotation_file_path)
        self.image_handler = ImageHandler(self.image_file_path)



import logging

import cv2


class ImageHandler:
    def __init__(self, image_file_path):
        """
        Constructor

        Parameters:
            image_file_path (pathlib.Path): Complete image file path
        """
        logging.debug("ImageHandler: ImageHandler initiated")
        if not image_file_path.exists():
            raise Exception(f"File {image_file_path} does not exist")

        self.image_file_path = image_file_path
        self.image_bgr = cv2.imread(str(image_file_path))
        self.image_rgb = cv2.cvtColor(self.image_bgr, cv2.COLOR_BGR2RGB)

    def get_image_name(self, include_extension=True):
        """
        Get the image file name

        Parameters:
            include_extension (bool): Whether to include image extension in file name
                Default=True
                Eg. Image name = image1.jpg
                    Include extension = True, returns image1.jpg
                    Include extension = False, returns image1
        """
        if include_extension:
            return self.image_file_path.name
        return self.image_file_path.stem

    def get_image_extension(self, include_dot=False):
        """
        Get the image extension

        Parameters:
            include_dot: Whether to return "jpg" or ".jpg"
                Default=False

        Eg. "image1.jpg" returns jpg, if include dot is True returns .jpg
            "image1.new.jpg" returns jpg, if include dot is True returns .jpg
        """
        if include_dot:
            return "." + self.image_file_path.name.rsplit(".", 1)[-1]
        return self.image_file_path.name.rsplit(".", 1)[-1]

    def get_bgr_image(self):
        """
        Returns the image in BGR format
        """
        return self.image_bgr

    def get_rgb_image(self):
        """
        Returns the image in RGB format
        """
        return self.image_rgb

    def save_image(self, image, save_path):
        """
        Save the image

        Parameters:
            image: The image to save
            save_path: Complete path of image where image need to be save

        Return:
            True if image saved successfully
            False if image save failed
        """
        logging.debug("ImageHandler: Save image %s", save_path)
        return cv2.imwrite(save_path, image)



from pascal_voc_toolkit.pascal_voc_dataset import PascalVocDataset
from pascal_voc_toolkit.pascal_voc_dataset_instance import PascalVocDatasetInstance
from pascal_voc_toolkit.xml_handler import XmlHandler
from pascal_voc_toolkit.image_handler import ImageHandler



import pathlib

from .pascal_voc_dataset_instance import PascalVocDatasetInstance


class PascalVocDataset:
    def __init__(self, image_dir_path: str, annotation_dir_path: str, image_file_format: str = ".jpg",
                 annotation_file_format: str = ".xml") -> None:
        self.image_dir_path: pathlib.Path = pathlib.Path(image_dir_path)
        self.annotation_dir_path: pathlib.Path = pathlib.Path(annotation_dir_path)
        self.image_file_format: str = image_file_format
        self.annotation_file_format: str = annotation_file_format

        # Check directories existence
        if not self.image_dir_path.exists():
            raise Exception(f"Specified image directory does not exist: {self.image_dir_path}")

        if not self.annotation_dir_path.exists():
            raise Exception(f"Specified annotation directory does not exist: {self.annotation_dir_path}")

        # Loading images and annotations
        self.image_files: list[pathlib.Path] = self._load_images()
        self.annotation_files: list[pathlib.Path] = self._load_annotations()
        self.dataset_instances: list[PascalVocDatasetInstance] = self.get_dataset_instances()

    def _load_images(self) -> list[pathlib.Path]:
        """
        Load images from the specified images directory
        """
        image_name_pattern: str = f"*{self.image_file_format}"
        return list(self.image_dir_path.glob(image_name_pattern))

    def _load_annotations(self) -> list[pathlib.Path]:
        """
        Load annotations from the specified annotations directory
        """
        annotation_name_pattern: str = f"*{self.annotation_file_format}"
        return list(self.annotation_dir_path.glob(annotation_name_pattern))

    def get_dataset_instances(self) -> list[PascalVocDatasetInstance]:
        """
        A list of PascalVocDatasetInstance which contains image and annotation file

        Returns:
            [ PascalVocDatasetInstance, ... ]
        """
        dataset_instances: list[PascalVocDatasetInstance] = []

        for image_file_path in self.image_files:
            image_name_without_extension = image_file_path.stem
            annotation_file_path = self.annotation_dir_path / (image_name_without_extension +
                                                               self.annotation_file_format)
            if annotation_file_path in self.annotation_files:
                dataset_instances.append(PascalVocDatasetInstance(image_file_path, annotation_file_path))
        return dataset_instances

    def get_labels(self) -> dict:
        labels: dict = {}
        count = 1
        for dataset_instance in self.dataset_instances:
            file_labels = dataset_instance.xml_handler.get_annotation_classes_details()
            for key, val in file_labels.items():
                if key in labels:
                    labels[key] += val
                else:
                    labels[key] = val
            print(f"Processing: {count} / {len(self.dataset_instances)}", end="\r")
            count += 1
        return labels

    def remove_labels(self, labels_to_remove: list[str]) -> None:
        count = 1
        for dataset_instance in self.dataset_instances:
            dataset_instance.xml_handler.remove_class(labels_to_remove)
            print(f"Processing: {count} / {len(self.dataset_instances)}", end="\r")
            count += 1

    def save_labels(self, output_directory: str = "output") -> None:
        # Creating output directory if not exists
        output_directory_path = pathlib.Path(output_directory)
        output_directory_path.mkdir(exist_ok=True, parents=True)

        count = 1
        for dataset_instance in self.dataset_instances:
            annotation_file_name = dataset_instance.xml_handler.get_xml_file_name(include_extension=True)
            dataset_instance.xml_handler.save_xml_file(output_directory_path / annotation_file_name)
            print(f"Processing: {count} / {len(self.dataset_instances)}", end="\r")
            count += 1



import pathlib

import pytest

from pascal_voc_toolkit import XmlHandler


# Pytest naming convention needs Test to prepend in class name
class TestXmlHandler:
    """
    Xml handler test class
    """

    def setup_class(self):
        """
        This will run once before test cases are run
        """
        self.current_file_path = pathlib.Path(__file__).resolve().parent
        self.invalid_file_path = pathlib.Path(
            self.current_file_path / "./res/invalid.xml")
        self.no_object_xml_file_path = pathlib.Path(
            self.current_file_path / "./res/no_annotated_object.xml")
        self.single_object_xml_file_path = pathlib.Path(
            self.current_file_path / "./res/single_annotated_object.xml")
        self.multiple_objects_xml_file_path = pathlib.Path(
            self.current_file_path / "./res/multiple_annotated_objects.xml")

    def test_xml_file_exists(self):
        """
        Test to check file existence
        """
        # Valid object
        XmlHandler(self.multiple_objects_xml_file_path)

        # Should throw exception for invalid file path
        with pytest.raises(Exception) as e_info:
            XmlHandler(self.invalid_file_path)

    def test_bounding_boxes_count_0(self):
        """
        Test case to check bounding boxes count for XML file with no object
        """
        no_object_xml_handler = XmlHandler(self.no_object_xml_file_path)
        assert no_object_xml_handler.get_bounding_boxes_count() == 0

    def test_bounding_boxes_count_1(self):
        """
        Test case to check bounding boxes count for XML file with single object
        """
        single_object_xml_handler = XmlHandler(self.single_object_xml_file_path)
        assert single_object_xml_handler.get_bounding_boxes_count() == 1

    def test_bounding_boxes_count_2(self):
        """
        Test case to check bounding boxes count for XML file with multiple objects
        """
        multiple_objects_xml_handler = XmlHandler(self.multiple_objects_xml_file_path)
        assert multiple_objects_xml_handler.get_bounding_boxes_count() == 87

    def test_xml_file_name_with_ext_0(self):
        """
        Test case to check file name including extension
        """
        xml_handler = XmlHandler(self.no_object_xml_file_path)
        assert xml_handler.get_xml_file_name(include_extension=True) == "no_annotated_object.xml"

    def test_xml_file_name_with_ext_1(self):
        """
        Test case to check file name including extension without passing any argument
        """
        xml_handler = XmlHandler(self.no_object_xml_file_path)
        assert xml_handler.get_xml_file_name() == "no_annotated_object.xml"

    def test_xml_file_name_without_ext(self):
        """
        Test case to check file name excuding extension
        """
        xml_handler = XmlHandler(self.no_object_xml_file_path)
        assert xml_handler.get_xml_file_name(include_extension=False) == "no_annotated_object"

    def test_annotation_classes_details_0(self):
        """
        Test case to check annotation classes details in XML file with no annotation boxes
        """
        xml_handler = XmlHandler(self.no_object_xml_file_path)
        annotated_class_details = xml_handler.get_annotation_classes_details()

        # Should be an empty dictionary
        # bool({}) will return False
        assert isinstance(annotated_class_details, dict)
        assert not bool(annotated_class_details)

    def test_annotation_classes_details_1(self):
        """
        Test case to check annotation classes details in XML file with single annotation boxes
        """
        expected_output_dict = {
            "person": 1
        }

        xml_handler = XmlHandler(self.single_object_xml_file_path)
        available_classes = xml_handler.get_annotation_classes_details()
        assert expected_output_dict == available_classes

    def test_annotation_classes_details_2(self):
        """
        Test case to check annotation classes details in XML file with multiple annotation boxes
        """
        expected_output_dict = {
            "person": 24,
            "small_vehicle": 13,
            "person_occluded_visible": 2,
            "heavy_vehicle": 48
        }

        xml_handler = XmlHandler(self.multiple_objects_xml_file_path)
        available_classes = xml_handler.get_annotation_classes_details()
        assert expected_output_dict == available_classes

    def test_bbox_coordinates_0(self):
        """
        Test case to check bounding box coordinates in XML file with no annotation boxes
        """
        xml_handler = XmlHandler(self.no_object_xml_file_path)
        bbox_details = xml_handler.get_all_bbox_coordinates_list()

        # The list should be empty
        assert isinstance(bbox_details, list)
        assert len(bbox_details) == 0

    def test_bbox_coordinates_1(self):
        """
        Test case to check bounding box coordinates in XML file with single annotation box
        """
        expected_output = [[676, 418, 693, 454, 'person']]

        xml_handler = XmlHandler(self.single_object_xml_file_path)
        bbox_details = xml_handler.get_all_bbox_coordinates_list()
        assert isinstance(bbox_details, list)
        assert expected_output == bbox_details

    def test_bbox_coordinates_2(self):
        """
        Test case to check bounding box coordinates in XML file with multiple annotation boxes
        """
        expected_output = [[651, 647, 673, 707, 'person'], [673, 651, 696, 703, 'person'],
                           [714, 450, 733, 483, 'person'],
                           [718, 467, 732, 498, 'small_vehicle'], [654, 370, 664, 393, 'person'],
                           [644, 364, 654, 391, 'person'],
                           [645, 365, 654, 382, 'person_occluded_visible'], [653, 354, 664, 385, 'person'],
                           [653, 354, 664, 373, 'person_occluded_visible'], [577, 204, 585, 220, 'person'],
                           [692, 180, 700, 197, 'person'],
                           [699, 127, 705, 139, 'person'], [904, 128, 914, 144, 'person'],
                           [1015, 156, 1024, 176, 'person'],
                           [121, 672, 147, 718, 'person'], [388, 686, 549, 761, 'heavy_vehicle'],
                           [428, 629, 556, 695, 'heavy_vehicle'],
                           [450, 566, 585, 643, 'heavy_vehicle'], [582, 581, 671, 692, 'heavy_vehicle'],
                           [604, 483, 661, 569, 'heavy_vehicle'],
                           [611, 443, 692, 500, 'heavy_vehicle'], [607, 390, 660, 444, 'heavy_vehicle'],
                           [469, 404, 518, 477, 'heavy_vehicle'],
                           [422, 310, 489, 348, 'heavy_vehicle'], [723, 282, 756, 324, 'heavy_vehicle'],
                           [740, 388, 787, 449, 'heavy_vehicle'],
                           [988, 464, 1054, 538, 'heavy_vehicle'], [1025, 549, 1101, 637, 'heavy_vehicle'],
                           [1063, 670, 1149, 765, 'heavy_vehicle'], [1136, 437, 1205, 494, 'heavy_vehicle'],
                           [1091, 370, 1162, 433, 'heavy_vehicle'], [1145, 333, 1235, 368, 'heavy_vehicle'],
                           [1122, 307, 1207, 340, 'heavy_vehicle'], [1101, 273, 1145, 310, 'heavy_vehicle'],
                           [895, 273, 929, 310, 'heavy_vehicle'], [871, 240, 909, 269, 'heavy_vehicle'],
                           [865, 184, 921, 209, 'heavy_vehicle'],
                           [950, 168, 1000, 191, 'heavy_vehicle'], [938, 161, 984, 180, 'heavy_vehicle'],
                           [914, 129, 959, 150, 'heavy_vehicle'],
                           [928, 119, 968, 139, 'heavy_vehicle'], [871, 103, 912, 120, 'heavy_vehicle'],
                           [648, 266, 677, 306, 'heavy_vehicle'],
                           [617, 241, 671, 272, 'heavy_vehicle'], [658, 195, 683, 223, 'heavy_vehicle'],
                           [726, 218, 752, 254, 'heavy_vehicle'],
                           [760, 139, 779, 158, 'heavy_vehicle'], [760, 126, 781, 142, 'heavy_vehicle'],
                           [797, 150, 819, 172, 'heavy_vehicle'],
                           [790, 129, 813, 149, 'heavy_vehicle'], [843, 158, 869, 187, 'heavy_vehicle'],
                           [731, 80, 766, 94, 'heavy_vehicle'],
                           [716, 49, 730, 68, 'heavy_vehicle'], [706, 71, 737, 90, 'heavy_vehicle'],
                           [487, 220, 557, 248, 'heavy_vehicle'],
                           [490, 211, 546, 232, 'heavy_vehicle'], [496, 196, 550, 222, 'heavy_vehicle'],
                           [505, 188, 558, 210, 'heavy_vehicle'],
                           [508, 181, 564, 204, 'heavy_vehicle'], [515, 170, 568, 194, 'heavy_vehicle'],
                           [529, 166, 578, 188, 'heavy_vehicle'],
                           [553, 119, 578, 147, 'heavy_vehicle'], [665, 142, 686, 166, 'heavy_vehicle'],
                           [186, 634, 236, 663, 'small_vehicle'],
                           [211, 608, 267, 642, 'small_vehicle'], [206, 561, 244, 606, 'small_vehicle'],
                           [326, 426, 361, 457, 'small_vehicle'],
                           [341, 411, 374, 435, 'small_vehicle'], [351, 386, 394, 412, 'small_vehicle'],
                           [376, 378, 405, 404, 'small_vehicle'],
                           [375, 360, 403, 384, 'small_vehicle'], [389, 350, 421, 368, 'small_vehicle'],
                           [429, 289, 457, 310, 'small_vehicle'],
                           [439, 279, 467, 294, 'small_vehicle'], [1089, 234, 1102, 254, 'small_vehicle'],
                           [375, 299, 386, 332, 'person'],
                           [384, 307, 396, 340, 'person'], [402, 312, 416, 344, 'person'],
                           [527, 144, 533, 158, 'person'],
                           [864, 244, 874, 268, 'person'], [854, 244, 867, 272, 'person'],
                           [870, 295, 888, 318, 'person'],
                           [725, 124, 733, 138, 'person'], [739, 110, 746, 125, 'person'],
                           [720, 97, 726, 109, 'person'],
                           [750, 111, 756, 124, 'person'], [706, 62, 712, 75, 'person']]

        xml_handler = XmlHandler(self.multiple_objects_xml_file_path)
        bbox_details = xml_handler.get_all_bbox_coordinates_list()
        assert isinstance(bbox_details, list)
        assert expected_output == bbox_details

    def test_update_bbox_coordinates_1(self):
        """
        Test case to check updating bounding box coordinates functionality in
        XML file with single annotation box
        """
        new_coordinates = [[647, 389, 664, 425, 'person_standing']]

        xml_handler = XmlHandler(self.single_object_xml_file_path)
        xml_handler.update_all_bounding_box_coordinates(new_coordinates)
        bbox_coordinates = xml_handler.get_all_bbox_coordinates_list()
        assert bbox_coordinates == new_coordinates

    def test_update_bbox_coordinates_2(self):
        """
        Test case to check updating bounding box coordinates functionality in
        XML file with multiple annotation boxes
        """
        new_coordinates = [[651, 647, 673, 707, 'person'], [673, 651, 696, 703, 'person'],
                           [714, 450, 733, 483, 'person'],
                           [718, 467, 732, 498, 'small_vehicle'], [654, 370, 664, 393, 'person'],
                           [644, 364, 654, 391, 'person'],
                           [645, 365, 654, 382, 'person_occluded_visible'], [653, 354, 664, 385, 'person'],
                           [653, 354, 664, 373, 'person_occluded_visible'], [577, 204, 585, 220, 'person'],
                           [692, 180, 700, 197, 'person'],
                           [699, 127, 705, 139, 'person'], [904, 128, 914, 144, 'person'],
                           [1015, 156, 1024, 176, 'person'],
                           [121, 672, 147, 718, 'person'], [342, 640, 503, 715, 'car'],
                           [428, 629, 556, 695, 'heavy_vehicle'],
                           [450, 566, 585, 643, 'heavy_vehicle'], [582, 581, 671, 692, 'heavy_vehicle'],
                           [604, 483, 661, 569, 'heavy_vehicle'],
                           [611, 443, 692, 500, 'heavy_vehicle'], [607, 390, 660, 444, 'heavy_vehicle'],
                           [469, 404, 518, 477, 'heavy_vehicle'],
                           [422, 310, 489, 348, 'heavy_vehicle'], [723, 282, 756, 324, 'heavy_vehicle'],
                           [740, 388, 787, 449, 'heavy_vehicle'],
                           [988, 464, 1054, 538, 'heavy_vehicle'], [1025, 549, 1101, 637, 'heavy_vehicle'],
                           [1063, 670, 1149, 765, 'heavy_vehicle'], [1136, 437, 1205, 494, 'heavy_vehicle'],
                           [1091, 370, 1162, 433, 'heavy_vehicle'], [1145, 333, 1235, 368, 'heavy_vehicle'],
                           [1122, 307, 1207, 340, 'heavy_vehicle'], [1101, 273, 1145, 310, 'heavy_vehicle'],
                           [895, 273, 929, 310, 'heavy_vehicle'], [871, 240, 909, 269, 'heavy_vehicle'],
                           [865, 184, 921, 209, 'heavy_vehicle'],
                           [950, 168, 1000, 191, 'heavy_vehicle'], [938, 161, 984, 180, 'heavy_vehicle'],
                           [914, 129, 959, 150, 'heavy_vehicle'],
                           [928, 119, 968, 139, 'heavy_vehicle'], [871, 103, 912, 120, 'heavy_vehicle'],
                           [648, 266, 677, 306, 'heavy_vehicle'],
                           [617, 241, 671, 272, 'heavy_vehicle'], [658, 195, 683, 223, 'heavy_vehicle'],
                           [726, 218, 752, 254, 'heavy_vehicle'],
                           [760, 139, 779, 158, 'heavy_vehicle'], [760, 126, 781, 142, 'heavy_vehicle'],
                           [797, 150, 819, 172, 'heavy_vehicle'],
                           [790, 129, 813, 149, 'heavy_vehicle'], [843, 158, 869, 187, 'heavy_vehicle'],
                           [731, 80, 766, 94, 'heavy_vehicle'],
                           [716, 49, 730, 68, 'heavy_vehicle'], [706, 71, 737, 90, 'heavy_vehicle'],
                           [487, 220, 557, 248, 'heavy_vehicle'],
                           [490, 211, 546, 232, 'heavy_vehicle'], [496, 196, 550, 222, 'heavy_vehicle'],
                           [505, 188, 558, 210, 'heavy_vehicle'],
                           [508, 181, 564, 204, 'heavy_vehicle'], [515, 170, 568, 194, 'heavy_vehicle'],
                           [529, 166, 578, 188, 'heavy_vehicle'],
                           [553, 119, 578, 147, 'heavy_vehicle'], [665, 142, 686, 166, 'heavy_vehicle'],
                           [186, 634, 236, 663, 'small_vehicle'],
                           [211, 608, 267, 642, 'small_vehicle'], [206, 561, 244, 606, 'small_vehicle'],
                           [326, 426, 361, 457, 'small_vehicle'],
                           [341, 411, 374, 435, 'small_vehicle'], [351, 386, 394, 412, 'small_vehicle'],
                           [376, 378, 405, 404, 'small_vehicle'],
                           [375, 360, 403, 384, 'small_vehicle'], [389, 350, 421, 368, 'small_vehicle'],
                           [429, 289, 457, 310, 'small_vehicle'],
                           [439, 279, 467, 294, 'small_vehicle'], [1089, 234, 1102, 254, 'small_vehicle'],
                           [375, 299, 386, 332, 'person'],
                           [384, 307, 396, 340, 'person'], [402, 312, 416, 344, 'person'],
                           [481, 98, 487, 112, 'person_sitting'],
                           [864, 244, 874, 268, 'person'], [854, 244, 867, 272, 'person'],
                           [870, 295, 888, 318, 'person'],
                           [725, 124, 733, 138, 'person'], [739, 110, 746, 125, 'person'],
                           [720, 97, 726, 109, 'person'],
                           [750, 111, 756, 124, 'person'], [706, 62, 712, 75, 'person']]

        xml_handler = XmlHandler(self.multiple_objects_xml_file_path)
        xml_handler.update_all_bounding_box_coordinates(new_coordinates)
        bbox_coordinates = xml_handler.get_all_bbox_coordinates_list()
        assert bbox_coordinates == new_coordinates

    def test_remove_class_1(self):
        """
        Test case to check removing classes functionality in XML file
        having single annotation box
        """
        expected_output = []
        xml_handler = XmlHandler(self.single_object_xml_file_path)
        xml_handler.remove_class(["person"])
        bbox_details = xml_handler.get_all_bbox_coordinates_list()
        assert bbox_details == expected_output

    def test_remove_classes_2(self):
        """
        Test case to check removing classes functionality in XML file
        having multiple annotation boxes
        """
        expected_output = [[651, 647, 673, 707, 'person'], [673, 651, 696, 703, 'person'],
                           [714, 450, 733, 483, 'person'],
                           [654, 370, 664, 393, 'person'], [644, 364, 654, 391, 'person'],
                           [645, 365, 654, 382, 'person_occluded_visible'],
                           [653, 354, 664, 385, 'person'], [653, 354, 664, 373, 'person_occluded_visible'],
                           [577, 204, 585, 220, 'person'],
                           [692, 180, 700, 197, 'person'], [699, 127, 705, 139, 'person'],
                           [904, 128, 914, 144, 'person'],
                           [1015, 156, 1024, 176, 'person'], [121, 672, 147, 718, 'person'],
                           [375, 299, 386, 332, 'person'],
                           [384, 307, 396, 340, 'person'], [402, 312, 416, 344, 'person'],
                           [527, 144, 533, 158, 'person'],
                           [864, 244, 874, 268, 'person'], [854, 244, 867, 272, 'person'],
                           [870, 295, 888, 318, 'person'],
                           [725, 124, 733, 138, 'person'], [739, 110, 746, 125, 'person'],
                           [720, 97, 726, 109, 'person'],
                           [750, 111, 756, 124, 'person'], [706, 62, 712, 75, 'person']]
        xml_handler = XmlHandler(self.multiple_objects_xml_file_path)
        xml_handler.remove_class(["small_vehicle", "heavy_vehicle"])
        bbox_coordinates = xml_handler.get_all_bbox_coordinates_list()
        assert bbox_coordinates == expected_output

    def test_save_file(self):
        """
        Test case to check file saving functionality
        """
        xml_file_path = pathlib.Path(
            self.current_file_path / "./res/multiple_annotated_objects.xml")

        output_directory = self.current_file_path / "generated"
        output_directory.mkdir(parents=True, exist_ok=True)
        output_file_path = output_directory / "output.xml"

        xml_handler = XmlHandler(xml_file_path)
        xml_handler.save_xml_file(str(output_file_path))
        assert output_file_path.exists()

        # Cleaning up
        output_file_path.unlink()
        output_directory.rmdir()



from codecs import open
from os import path

from setuptools import setup

# Package Version
VERSION = '1.0.1'

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pascal-voc-toolkit',
    version=VERSION,
    packages=['tests', 'pascal_voc_toolkit'],
    url='',
    license='',
    author='Aman Khatri',
    author_email='amankhatri.ai@gmail.com',
    description='Set of tools to work with Pascal VOC annotation format',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['dict2xml==1.7.2', 'xmltodict==0.13.0', 'pytest==7.2.1', 'opencv-python==4.7.0.68']
)
